@@ -1,5 +1,9 @@
 #![no_std]
 
+//! This library provides an array type that is similar to the built-in Vec type, but lives on the stack!
+//!
+//! You can store a fixed number of elements of a specific type (even non-copy types!)
+
 use core::{
     fmt,
     mem::{replace, MaybeUninit},
@@ -7,11 +11,12 @@ use core::{
     ptr,
 };
 
-/// SEAFTY: Caller must ensure that `value` is properly initialized.
+/// Caller must ensure that `value` is properly initialized.
 unsafe fn take<T>(dest: &mut MaybeUninit<T>) -> T {
     replace(dest, MaybeUninit::uninit()).assume_init()
 }
 
+/// A data structure for storing and manipulating fixed number of elements of a specific type.
 pub struct Array<T, const N: usize> {
     len: usize,
     data: [MaybeUninit<T>; N],
@@ -19,112 +24,146 @@ pub struct Array<T, const N: usize> {
 
 impl<T, const N: usize> Array<T, N> {
     /// Creates a new [`Array<T, N>`].
-    /// 
+    ///
     /// # Examples
-    /// 
+    ///
     /// ```
     /// use stack_array::Array;
-    /// 
-    /// let array: Array<u8, 4> = Array::new();
+    ///
+    /// let arr = Array::<u8, 4>::new();
     /// // or
-    /// let array = Array::<u8, 4>::new();
+    /// let arr: Array<u8, 4> = Array::new();
     /// ```
-    pub fn new() -> Self {
-        Self::default()
+    pub const fn new() -> Self {
+        Self {
+            len: 0,
+            // SAFETY: An uninitialized `[MaybeUninit<_>; N]` is valid.
+            data: unsafe { MaybeUninit::uninit().assume_init() },
+        }
     }
 
-    
     /// Returns the number of elements the array can hold.
-    /// 
+    ///
     /// # Examples
     ///
     /// ```
     /// use stack_array::Array;
     ///
-    /// let array: Array<u8, 4> = Array::new();
-    /// assert_eq!(array.capacity(), 4);
+    /// let arr: Array<u8, 4> = Array::new();
+    /// assert_eq!(arr.capacity(), 4);
     /// ```
-    pub fn capacity(&self) -> usize {
+    #[inline(always)]
+    pub const fn capacity(&self) -> usize {
         N
     }
 
     /// Returns the number of elements currently in the array.
-    /// 
+    ///
     /// # Examples
-    /// 
+    ///
     /// ```
     /// use stack_array::Array;
     ///     
-    /// let mut array: Array<u8, 3> = Array::from([1, 2]);
-    /// assert_eq!(array.len(), 2);
+    /// let arr: Array<u8, 3> = Array::from([1, 2]);
+    /// assert_eq!(arr.len(), 2);
     /// ```
-    pub fn len(&self) -> usize {
+    #[inline(always)]
+    pub const fn len(&self) -> usize {
         self.len
     }
 
+    /// Returns `true`, If the array is full.
+    /// 
     /// # Examples
     ///
     /// ```
     /// use stack_array::Array;
     ///
-    /// let mut array: Array<u8, 3> = Array::from([1, 2]);
-    /// assert!(!array.is_full());
+    /// let arr: Array<u8, 3> = Array::from([1, 2]);
+    /// assert!(!arr.is_full());
     /// ```
-    pub fn is_full(&self) -> bool {
-        self.len >= N
+    #[inline(always)]
+    pub const fn is_full(&self) -> bool {
+        N == self.len
+    }
+
+    /// Returns true if the vector contains no elements.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use stack_array::Array;
+    ///
+    /// let mut arr: Array<u8, 2> = Array::new();
+    /// assert!(arr.is_empty());
+    /// 
+    /// arr.push(1);
+    /// assert!(!arr.is_empty());
+    /// ```
+    #[inline(always)]
+    pub const fn is_empty(&self) -> bool {
+        self.len == 0
     }
 
     /// Returns the number of elements can be inserted into the array.
-    /// 
+    ///
     /// # Examples
     ///
     /// ```
     /// use stack_array::Array;
     ///
-    /// let mut array: Array<u8, 3> = Array::from([1, 2]);
-    /// assert_eq!(array.remaing(), 1);
+    /// let arr: Array<u8, 3> = Array::from([1, 2]);
+    /// assert_eq!(arr.remaining_capacity(), 1);
     /// ```
-    pub fn remaing(&self) -> usize {
+    #[inline(always)]
+    pub const fn remaining_capacity(&self) -> usize {
         N - self.len
     }
 
-
     /// Appends an element to the back of a collection
-    /// 
+    ///
     /// ### Examples
-    /// 
+    ///
     /// ```rust
     /// use stack_array::Array;
     ///
-    /// let mut list: Array<u8, 3> = Array::from([1]);
-    /// list.push(2);
-    /// list.push(3);
-    /// assert_eq!(&list[..], [1, 2, 3]);
+    /// let mut arr: Array<u8, 3> = Array::from([1]);
+    /// arr.push(2);
+    /// arr.push(3);
+    /// assert_eq!(&arr[..], [1, 2, 3]);
     /// ```
+    /// 
+    /// # Panics
+    /// Panics if the array is full.
+    #[inline(always)]
     pub fn push(&mut self, val: T) {
         self.data[self.len] = MaybeUninit::new(val);
         self.len += 1;
     }
 
     /// Removes the last element from a collection and returns it.
-    /// 
+    ///
     /// # Examples
-    /// 
+    ///
     /// ```rust
     /// use stack_array::Array;
-    /// 
-    /// let mut list: Array<u8, 3> = Array::from([1, 2]);
-    /// assert_eq!(list.pop(), 2);
-    /// assert_eq!(list.pop(), 1);
-    /// assert_eq!(list.len(), 0);
+    ///
+    /// let mut arr: Array<u8, 3> = Array::from([1, 2]);
+    /// assert_eq!(arr.pop(), 2);
+    /// assert_eq!(arr.pop(), 1);
+    /// assert!(arr.is_empty());
     /// ```
+    /// 
+    /// # Panics
+    /// Panics if the array is empty.
+    #[inline(always)]
     pub fn pop(&mut self) -> T {
         self.len -= 1;
         unsafe { take(&mut self.data[self.len]) }
     }
 
     /// Clears the array, removing all values.
-    /// 
+    ///
     /// # Examples
     ///
     /// ```
@@ -156,6 +195,10 @@ impl<T, const N: usize> Array<T, N> {
     /// list.insert(1, 2);
     /// assert_eq!(&list[..], [1, 2, 3]);
     /// ```
+    /// 
+    /// # Panics
+    /// Panics if the index is out of bounds.
+    #[inline]
     pub fn insert(&mut self, index: usize, val: T) {
         assert!(index <= self.len);
         for i in (index..self.len).rev() {
@@ -166,7 +209,7 @@ impl<T, const N: usize> Array<T, N> {
     }
 
     /// Removes an element from position index within the array, shifting all elements after it to the left.
-    /// 
+    ///
     /// # Examples
     ///
     /// ```
@@ -177,6 +220,10 @@ impl<T, const N: usize> Array<T, N> {
     /// assert_eq!(list.remove(0), 2);
     /// assert_eq!(list.remove(0), 3);
     /// ```
+    /// 
+    /// # Panics
+    /// Panics if the index is out of bounds.
+    #[inline]
     pub fn remove(&mut self, index: usize) -> T {
         assert!(index < self.len);
         let value = unsafe { take(&mut self.data[index]) };
@@ -190,38 +237,39 @@ impl<T, const N: usize> Array<T, N> {
 
 impl<T, const N: usize> Default for Array<T, N> {
     fn default() -> Self {
-        Self {
-            len: 0,
-            data: unsafe { MaybeUninit::uninit().assume_init() },
-        }
+        Self::new()
     }
 }
 
 impl<T, const N: usize> AsRef<[T]> for Array<T, N> {
+    #[inline]
     fn as_ref(&self) -> &[T] {
         // unsafe { core::mem::transmute(&self.data[..self.len]) }
         // unsafe { slice::from_raw_parts(self.data.as_ptr() as *const _, self.len) }
 
-        // SAFETY: `self.data[..self.len]` is initialized.
+        // SAFETY: slice will contain only initialized objects.
         unsafe { &*(&self.data[..self.len] as *const [MaybeUninit<T>] as *const [T]) }
     }
 }
 
 impl<T, const N: usize> AsMut<[T]> for Array<T, N> {
+    #[inline]
     fn as_mut(&mut self) -> &mut [T] {
-        // SAFETY: `self.data[..self.len]` is initialized.
+        // SAFETY: slice will contain only initialized objects.
         unsafe { &mut *(&mut self.data[..self.len] as *mut [MaybeUninit<T>] as *mut [T]) }
     }
 }
 
 impl<T, const N: usize> Deref for Array<T, N> {
     type Target = [T];
+    #[inline]
     fn deref(&self) -> &Self::Target {
         self.as_ref()
     }
 }
 
 impl<T, const N: usize> DerefMut for Array<T, N> {
+    #[inline]
     fn deref_mut(&mut self) -> &mut Self::Target {
         self.as_mut()
     }
@@ -260,4 +308,4 @@ impl<T, const N: usize, const S: usize> From<[T; S]> for Array<T, N> {
         }
         array
     }
-}
\ No newline at end of file
+}
